# -*- coding: UTF-8 -*-
#
# Получение сообщений из очереди
#

from artix.common import py2to3utils
from plugins import status
from qpid.messaging import *
import logger

loggerSync = logger.getLogger()
messageTypeKey = 'type'
goodMessageFile = 'lastGoodMessage.txt'


def saveLastGoodMessageId(messageId):
    try:
        with open(goodMessageFile, 'w') as outfile:
            outfile.write(messageId)
    except Exception as e:
        loggerSync.warn(e)


def getLastGoodMessageId():
    try:
        with open(goodMessageFile, 'r') as infile:
            for line in infile:
                if line and line.strip():
                    return line
    except IOError as e:
        loggerSync.warn(e)
    return None


def main(properties):
    res = status.OK  # результат. Если проверка очереди прошла нормально, то вернется 1
    mess = ''
    try:
        loggerSync.info('Создаем соединение')
        connection = Connection(properties.remoteBroker, heartbeat=properties.brokerConnectionHeartbeat)
        connection.tcp_nodelay = True
        loggerSync.info('Открываем соединение')
        connection.open()
    except Exception as e:
        loggerSync.error(str(e))
        return status.errorProgramm, str(e)
    try:
        loggerSync.info('Создаем сессию')
        session = connection.session()
        loggerSync.info('Создаем получателя сообщений')
        destination = "artix.forward.cs" + "." + properties.cashCode
        receiver = session.receiver(destination)
        while True:
            loggerSync.info('Проверяем наличие сообщений в очереди')
            message = receiver.fetch(timeout=1)
            messageId = str(message.id) if message.id else None
            if messageId and messageId == getLastGoodMessageId():
                loggerSync.info('Сообщение с идентификатором %s уже было обработано. Удаляем его.' % messageId)
                session.acknowledge()
                continue

            loggerSync.info('Получили сообщение')
            messageType = message.properties.get(messageTypeKey).lower()
            if not messageType:
                loggerSync.error("Тип сообщения не указан")
                continue
            try:
                plugin = __import__('plugins.%s' % messageType, fromlist='q')
                # выполняем скрипт, соответствующий типу сообщения
                loggerSync.info('Вызываем плагин %s для обработки сообщения' % py2to3utils.ensureStr(messageType))
                try:
                    res, mess = plugin.main(message)
                except Exception as e:
                    mess = str(e)
                    res = status.errorProgramm

                if res != status.OK:
                    receiver.close()
                    session.close()
                    connection.close()
                    return res, mess

                loggerSync.info('Удаляем из очереди принятое сообщение %s' % messageId)
                saveLastGoodMessageId(messageId)
                session.acknowledge()
            except ImportError as e:
                res = status.errorProgramm
                mess = "Неизвестный тип сообщения. " + str(e)
                loggerSync.error(mess)
            loggerSync.info('')
    except NotFound as e:
        res = status.errorProgramm
        mess = str(e)
        loggerSync.error(str(e))
    except Empty:
        mess = 'Проверка сообщений закончена'
        loggerSync.info(mess)
        res = status.OK
        receiver.close()
    finally:
        loggerSync.info('-----------------------')
        if session:
            session.close()
        connection.close()
    return res, mess


if __name__ == '__main__':
    from propertyReader import properties
    main(properties)
